#ifndef TYPETAGGER_CMD_OPTIONS_HPP
#define TYPETAGGER_CMD_OPTIONS_HPP

#include <boost/program_options.hpp>
#include <boost/tuple/tuple.hpp>

namespace typetagger {

boost::tuple<boost::program_options::options_description,
             boost::program_options::options_description>
make_options()
{
  using namespace std;
  namespace po = boost::program_options;
  po::options_description config("Configuration (both command line and config file)");
  config.add_options()
    ("random-seed,r", po::value<unsigned int>(), "set random seed")
    //("use-features,f", "use features")
    //("prior,p", "use type-level tag prior")
    ("iterations,i", po::value<size_type>(), "number of iterations")
    ("num-states,k", po::value<size_type>(), "number of states")    
    ("wseq", po::value<string>(), ".wseq filename")
    ("wlex", po::value<string>(), ".wlex filename")
    ("tseq", po::value<string>(), ".tseq filename")
    ("tlex", po::value<string>(), ".tlex filename")
    ("wflex", po::value<string>(), ".wflex filename")
    ("wfeats", po::value<string>(), ".wfeats filename")
    ("outfile,o", po::value<string>(), "output tseq filename")
    ("alpha-emission", po::value<float_type>(), "hyperparameter for emission")
    ("alpha-transition", po::value<float_type>(), "hyperparameter for transition")
    ("alpha-tag", po::value<float_type>(), "hyperparameter for (type-level) tag prior")
    ("alpha-features", po::value<float_type>(), "hyperparameter for (type-level) features")
    ("approx", "approximate inference")
    ("dict,d", po::value<string>(), ".dict filename")
    ("init", po::value<int>(), "initialization: 0-freq heuristic 1-random")
    ("resample,s", po::value<int>(), "resample hyperparameters every s iteration. if s<0, init from prior")    

    ("num-closed-states", po::value<size_type>(), "number of closed states")
    ("use-gtree,t", "use gtree")
    ("alpha-gtree", po::value<float_type>(), "hyperparameter for gtree")
    ("init-dict", po::value<string>(), ".dict filename")
    ("tclass", po::value<string>(), ".toc filename")
    ("wstem", po::value<string>(), ".wstem filename")
    ("wgroup", po::value<string>(), ".wgroup filename")
    ("use-dict,d", "use partial lexicon dict during learning (optional)")
    ("use-init-dict", "use lexicon for initialization (optional)")
    ("use-open-closed,c", "use open/closed class distinction")
    ("gtree", po::value<string>(), ".gtree filename")
    ;

  po::options_description cmdline("Command line only");
  cmdline.add_options()
    ("help,h", "print help message")
    ("config", po::value<string>(), "config file (command line takes precedence)")
    ;
  cmdline.add(config);

  return boost::make_tuple(cmdline,config);
}

char** strdup_args(int argc, const char* const argv[])
{
  char** a = (char**) malloc(sizeof(char*) * argc);
  for(int i = 0; i < argc; ++i)
    {
      a[i] = strdup(argv[i]);
    }
  return a;
}

void free_args(int argc, char* argv[])
{
  for(int i = 0; i < argc; ++i)
    {
      free(argv[i]);
    }
  free(argv);
}

boost::program_options::variables_map
parse_options(const boost::program_options::options_description& cmdline,
              const boost::program_options::options_description& config,
              const int argc,
              const char* const argv[])
{
  namespace po = boost::program_options;
  
  char** argv_copy = strdup_args(argc, argv);
  char** argv_copy2 = strdup_args(argc, argv);

  po::variables_map vm1;
  po::store(po::parse_command_line(argc, argv_copy, cmdline),vm1);
  po::notify(vm1);

  po::variables_map vm;
  po::store(po::parse_command_line(argc, argv_copy2, cmdline),vm);
  if (vm1.count("config"))
    {
      const std::string cfn = vm1["config"].as<std::string>();
      boost::scoped_ptr<std::ifstream> infile( safe_ifopen(cfn.c_str()) );
      std::cout << "config = " << cfn << std::endl;
      po::store(po::parse_config_file(*infile, config),vm);
      infile->close();
    }
  po::notify(vm);
  
  free_args(argc, argv_copy);
  free_args(argc, argv_copy2);

  return vm;
}

} // namespace 

#endif // TYPETAGGER_CMD_OPTIONS_HPP
