import sys
import os
import re
import string
import collections
import gzip
import pdb

import run_morfessor_catmap
import make_utf8_morfessor_valid
    
def make_morfessor_compliant ( words, charmap ):
    compliant_words = tuple ( "".join ( charmap[c] for c in w )
                            for w in words )
    return compliant_words

def get_suffixes ( words, tmpdir, wseq_filename, morfessor_bindir ):
    charset = set ( c
                   for w in words
                   for c in w )
    charmap = make_utf8_morfessor_valid. get_utf8_to_8bit_map_from_set ( charset )
    if charmap == None:
        # can't run morfessor
        return tuple ( "" for w in words )

    if not os.path.isdir(tmpdir): os.mkdir (tmpdir)

    # create input data file    
    wordcounts = collections.Counter ( wid
                                      for line in open(wseq_filename)
                                      for wid in line.strip().split() )

    compliant_words = make_morfessor_compliant ( words, charmap )
    
    outfile = gzip.open(os.path.join(tmpdir,"mydata.gz"),'w+')
    for wid,count in wordcounts.iteritems():
        print >>outfile, count, compliant_words [int(wid)-1]
        #print count, compliant_words [int(wid)-1]
    outfile.close()

    # run morfessor_catmap
    run_morfessor_catmap. write_makefile ( morfessor_bindir, open(os.path.join(tmpdir,"Makefile"),'w+') )
    saved_dir = os.getcwd()
    os.chdir ( tmpdir )
    os.environ["LC_ALL"] = "POSIX" # make sort -u, uniq in Morfessor Makefile work properly
    os.system ( "make realclean" )
    os.system ( "make" )
    segmentations = run_morfessor_catmap. read_segmentations ( gzip.open("segmentation.final.gz") )
    os.chdir ( saved_dir )

    # get suffixes at the end of each segmentation
    suffix_dict = {}
    for s in segmentations:
        word = "".join(m for i,m in enumerate(s) if i % 2 == 0)
        suffixes = []
        rs = tuple(reversed(s))
        for i,m in enumerate(rs):
            if i % 2 == 0:
                if m == "SUF":
                    suffixes. append ( rs[i+1] )
                else:
                    break
        suffix_dict [ word ] = "".join(reversed(suffixes)) if len(suffixes) > 0 else ""

    d = {}
    for w in compliant_words:
        d[w] = suffix_dict [ w ]

    assert len(words) == len(compliant_words)
    rcharmap = dict ( map ( reversed, charmap.iteritems()) )
    suffixes = []
    for w,wm in zip(words,compliant_words):
        suffix_ascii = d[wm]
        suffix_utf8 = u"".join( rcharmap[c] for c in suffix_ascii)
        suffixes. append ( suffix_utf8 )
    return tuple(suffixes)

def main():
    # python make_wordtype_features.py ../data/conllx.swedish.train.{wlex,wseq} tmp $HOME/research/morfessor_catmap0.9.2/bin tmp/swedish.feats tmp/swedish.feats.lex
    wlex_filename = sys.argv[1] # UTF-8; might be non-morfessor compliant
    wseq_filename = sys.argv[2]
    tmpdir = sys.argv[3]
    morfessor_bindir = sys.argv[4]
    wfeats_filename = sys.argv[5]
    wfeatslex_filename = sys.argv[6]

    words = tuple ( line.strip().decode("utf-8") for line in open(wlex_filename) )    

    # get suffixes from morfessor
    words_suffixes = get_suffixes ( words, tmpdir, wseq_filename, morfessor_bindir )
    assert len(words_suffixes) == len(words)

    # create feature files
    wlexes = collections. defaultdict ( dict )
    outfile = open(wfeats_filename,'w')
    for i,w in enumerate(words):
        features = []
        features. append ( "has-digits:1" if re.search('\d',w) else "has-digits:0" )
        features. append ( "has-punct:1" if any(p in w for p in string.punctuation) else "has-punct:0" )
        features. append ( "init-cap:1" if w[0].isupper()  else "init-cap:0" )
        # for i in range(3):
        #     if len(w)-i-1 > 0:
        #         features. append ( "suffix%d:%s" % (i+1,w[-1-i:]) )
        #     else:
        #         features. append ( "suffix%d:" % (i+1) )
        # #print "%s %s" % ( w, " ".join([" ".join(f) for f in features] ) )
        features.append ( "suffix:%s" % words_suffixes[i] )
        fvector = [None] * len(features)
        for i,f in enumerate(features):
            if f in wlexes[i]:
                fvector [i] = wlexes[i][f]
            else:
                n = len(wlexes[i]) + 0
                wlexes[i][f] = n
                fvector [i] = n

        #print w, " ".join(str(i) for i in fvector)
        print >>outfile, " ".join(str(i) for i in fvector)
        #pdb.set_trace()

    outfile.close()
    outfile = open(wfeatslex_filename,'w')
    for i,wlex in sorted(wlexes.iteritems(),key=lambda (i,wlex):i):
        for k,v in sorted(wlex.iteritems(),key=lambda (k,v): v):
            print >>outfile,k.encode("utf-8"),
        print >>outfile,""
    outfile.close()
    
if __name__ == '__main__':
    main()
