#!/bin/bash

set -e

usage="Usage: $0 input_sentences num_states model_number output_dir"
infile=$1
num_states=$2
m=$3 # model 1 = basic, 2 = +prior, 3 = +feats (full model)
d=$4 # output dir

if [ $# -lt 4 ] || [ $2 -lt 1 ] || [ $3 -lt 1 ] || [ $3 -gt 3 ]; then
    echo $usage
    exit 1
fi

iters=50
rseed=0
resampleits=10
aemit=0.01
atrans=1.0
atag=0.01
afeats=0.01
model3_args=""
b=$(dirname $0) # basedir
bb=$(dirname $(readlink -e $0))
MORFESSOR_BINDIR=$bb/morfessor_catmap0.9.2-bin # needs ABSOLUTE path

## creates output directory
#if [ -d $d ]; then rm -rf $d; fi
if [ ! -d $d ]; then mkdir -p $d; fi

## creates input data files which represent words and tags as integers
cp -a $infile $d/words.txt
python $b/make_dummy_tags.py  < $infile > $d/tags.txt
python $b/make_seq_files.py $d/{words,tags}.txt $d/data.{wseq,wlex,tseq,tlex}

## set arguments for the specified model
if [ $m -eq 1 ]; then
    atag=-1
    afeats=-1
elif [ $m -eq 2 ]; then
    afeats=-1
elif [ $m -eq 3 ]; then
    # needs morfessor to get suffix feature
    echo "> python $b/make_wordtype_features.py $d/data.{wlex,wseq} $d/tmp $MORFESSOR_BINDIR $d/data.{wfeats,wflex}"
    python $b/make_wordtype_features.py $d/data.{wlex,wseq} $d/tmp $MORFESSOR_BINDIR $d/data.{wfeats,wflex}
    model3_args="--wfeats $d/data.wfeats"
fi
outfile=$d/$m.dict

## make configuration file
python $b/make_cfg.py --wseq $d/data.wseq \
    --tseq $d/data.tseq \
    --wlex $d/data.wlex \
    --tlex $d/data.tlex \
    -k $num_states \
    -i $iters \
    -r $rseed \
    --alpha-emission $aemit\
    --alpha-transition $atrans \
    --alpha-tag $atag\
    --alpha-features $afeats \
    --outfile $d/$m.d \
    --init 1 \
    --resample $resampleits \
    $model3_args \
    > $d/$m.config

## run the C++ binary
$b/run_typetagger --config $d/$m.config 

## lexicalize learned tagging dictionary
python $b/lexicalize_dict.py $d/data.wlex $d/$m.d > $outfile

