#include <cstdlib>
#include <iostream>
#include <cassert>
#include <iterator> // ostream_iterator
#include <algorithm> // copy
#include <boost/foreach.hpp>

#include "type_config.hpp"
#include "dataio.hpp"
#include "bhmm.hpp"
#include "cmd_options.hpp"

#include "utilities.hpp"


using namespace std;
namespace po = boost::program_options;

int main(int argc, char* argv[])
{

  boost::tuple<po::options_description,po::options_description> options_tuple = typetagger::make_options();
  po::variables_map vm = typetagger::parse_options(options_tuple.get<0>(), options_tuple.get<1>(), argc, argv);

  if (vm.count("help"))
    {
      cout << options_tuple.get<0>() << std::endl;
      return 0;
    }

  const char* const wseq_filename = vm["wseq"].as<string>().c_str();
  const char* const tseq_filename = vm["tseq"].as<string>().c_str();
  const char* const wlex_filename = vm["wlex"].as<string>().c_str();
  const char* const tlex_filename = vm["tlex"].as<string>().c_str();
  const char* const wfeats_filename = vm.count("wfeats") ? vm["wfeats"].as<string>().c_str() : 0;
  const char* const dict_filename = vm.count("dict") ? vm["dict"].as<string>().c_str() : 0;

  const unsigned int s = vm["random-seed"].as<unsigned int>();
  typetagger::boost_rng_type rng_engine;
  rng_engine.seed(s);
  //srandom(s);

  typetagger::bhmm h(wseq_filename,
                     tseq_filename,
                     wlex_filename,
                     tlex_filename,
                     wfeats_filename,
                     dict_filename);
  const typetagger::bhmm_gibbs_options opts(vm);

  cout << "random-seed " << s << endl;
  cout << opts << endl;

  const typetagger::lexicon_state state = h.run_gibbs(opts,rng_engine);
  if (vm.count("outfile"))
    {
      cout << "outfile = " << vm["outfile"].as<string>().c_str() << endl;
      typetagger::dump_tagging_dict<typetagger::size_type>( vm["outfile"].as<string>().c_str(), state.tagging_dict() );
    }

  return 0;
}
